#version 330

uniform sampler2D texUnit;

out vec4 outputF;

//----------------------------------------------------------------
// TODO:
// 1) Blur the image by averaging the 9 values (8-neighbourhood).
// 1.1) Iterate throught texels
// 1.2) Protect against the out-of-range accesses 
// 1.3) Fetch texel data
// 1.4) sum values
// 1.5) Divide sum

// 2) Improve code for arbitrary blurring size
// 3) Blur the image with Gaussian blur kernel.
//----------------------------------------------------------------

//----------------------------------------------------------------
// Problems:
// 1) Darkening at image borders. There is simple solution... find it :-)
// 2) Gaussian - need for various coeficients.
//----------------------------------------------------------------

void main()
{
	ivec2 texSize = textureSize(texUnit, 0);
	ivec2 pos = ivec2(gl_FragCoord);
	ivec2 offset;
	vec4 sum = vec4(0.f);
	
	//only odd numbers will work correctly
	int kernelSize = 11;
	int range = (kernelSize-1) / 2;


// Blur with chosen kernel
#if 1

	//TODO 1.1) Finish for statements
	for(offset.y = -range; offset.y <= range; offset.y++)
	{
		for(offset.x = -range; offset.x <= range; offset.x++)
		{
			//TODO 1.2) Skip texels "outside" the texture
			//   hint - check 'pos+offset' value...
			//   hint - use 'any()', 'lessThan()', 'greaterThan()'  functions
			//   hint - use 'textureSize()' function
			if(  any( lessThan(pos+offset, ivec2(0)) || greaterThan(pos+offset, texSize) ) )
			{
				//outside the texture, just skip
				continue;
			}

			//TODO 1.3) get texel data
			vec4 texel = texelFetch(texUnit, pos+offset, 0);

			//TODO 1.4) sum it
			sum += texel;
		}
	}
	
	//TODO 1.5) divide the sum by number of summed texels
	outputF = sum / (kernelSize*kernelSize);
#endif




// Blur with Gaussian kernel
#if 0

	// Gauss 3x3
	kernelSize = 3;
	range = 1;
	float weights[9] = float[9](1.f, 2.f, 1.f,
								2.f, 4.f, 2.f,
								1.f, 2.f, 1.f);

	 
/*
	// Gauss 5x5
	kernelSize = 5;
	range = 2;
	float weights[25] = float[25](2.f, 7.f, 12.f, 7.f, 2.f,
								7.f, 31.f, 52.f, 31.f, 7.f,
								15.f, 52.f, 127.f, 52.f, 15.f,
								7.f, 31.f, 52.f, 31.f, 7.f,
								2.f, 7.f, 12.f, 7.f, 2.f);
*/  

	float sumWeight = 0.f;

	for(offset.y = -range; offset.y <= range; offset.y++)
	{
		for(offset.x = -range; offset.x <= range; offset.x++)
		{
			// Skip texels "outside" the texture
			if(any(lessThan(pos+offset, ivec2(0)) || greaterThan(pos+offset, texSize)))
			{
				continue;
			}

			vec4 texel = texelFetch(texUnit, pos+offset, 0);

			//different range, different offsets
			float weight = weights[(offset.y+range)*kernelSize+(offset.x+range)];
			sum += texel * weight;
			sumWeight += weight;
		}
	}
	
	outputF = sum/sumWeight;
#endif
}