#version 330

in vec2 texCoord;

uniform sampler2D texUnit;
uniform sampler2D kernelUnit;
uniform ivec2 kernelSize = ivec2(5, 5);
uniform bool grayscale = false;
uniform bool normalize = true;

out vec4 outputF;

//----------------------------------------------------------------
// TODO:
// 1) Add uniforms for:
//		- second sampler, 
//		- kernel size,
//		- grayscale
//		- normalize

// 2) Return to using the texture function to allow wrapped out-of-range accesses.

// 3) Multiply the kernel with the neighbourhood of each pixel.
//		- Load the texel
//		- If required, convert its value to grayscale.
//		- Load the weights from the red channel.
//		- Normalize the sum by the sum of weights only if required.
// 4) Compute the convolved image.
//----------------------------------------------------------------

//----------------------------------------------------------------
// Solved problems:
// 1) Blackening around the edges.
// 2) Different codes for different sizes.
// 3) Need for various coeficients.
//----------------------------------------------------------------

void main()
{
	ivec2 texSize = textureSize(texUnit, 0);
	ivec2 offset;
	ivec2 kernelSizeHalf = kernelSize/2;
	vec2 scale = 1.f / vec2(texSize);
	vec4 sum = vec4(0.f);
	float sumWeight = 0.f;

	for(offset.y = -kernelSizeHalf.y; offset.y <= kernelSizeHalf.y; offset.y++)
	{
		for(offset.x = -kernelSizeHalf.x; offset.x <= kernelSizeHalf.x; offset.x++)
		{
			vec4 texel;
			if(grayscale)
			{
				vec3 NTSCWeights = vec3(0.299f, 0.587f, 0.114f);
				vec4 tex = texture(texUnit, texCoord + offset*scale, 0);
				float gray = dot(tex.rgb, NTSCWeights);
				texel = vec4(gray, gray, gray, tex.a);
			}
			else
			{
				texel = texture(texUnit, texCoord + offset*scale, 0);
			}

			float weight = texelFetch(kernelUnit, offset+kernelSizeHalf, 0).r; // Beware coordinates order
			sum += texel * weight;
			sumWeight += weight;
		}
	}
	
	if(normalize)
	{
		outputF = sum/sumWeight;
	}
	else
	{
		outputF = sum;
	}
}